// ignore_for_file: camel_case_types
import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:nim_fluttify/src/android/android.export.g.dart';
import 'package:nim_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';
import 'extensions.dart';
import 'models.dart';

part 'callbacks.dart';

class Nim
    with
        _NimStream,
        _Auth,
        _SendMessage,
        _UserRelationship,
        _ChatRoom,
        _Session,
        _HistoryMessage {
  static Nim instance = Nim._();

  Nim._();

  /// 初始化
  Future<void> init(String appKey) async {
    debugPrint('此为付费插件, 请联系qq 382146139');

    // 注册自定义消息解析类
    await MethodChannel('com.fluttify/nim_fluttify')
        .invokeMethod('attachCustomParser');
  }
}

/// 流监听
mixin _NimStream {
  final _onlineStatus = StreamController<StatusType>.broadcast();
  final _message = StreamController<Message>.broadcast();
  final _chatRoomStatus = StreamController<ChatRoomStatus>.broadcast();
  final _kickOut = StreamController<KickOut>.broadcast();
  final _systemMessage = StreamController<SystemMessage>.broadcast();

  // 是否已经设置监听, 防止多次登录导致多次设置监听
  bool _alreadySetup = false;

  /// 消息流
  Stream<Message> get messageStream => _message.stream;

  /// 在线状态流
  Stream<StatusType> get onlineStatusStream => _onlineStatus.stream;

  /// 聊天室状态流
  Stream<ChatRoomStatus> get chatRoomEventStream => _chatRoomStatus.stream;

  /// 聊天室踢出状态流
  Stream<KickOut> get kickOutStream => _kickOut.stream;

  /// 系统消息流
  Stream<SystemMessage> get systemMessageStream => _systemMessage.stream;

  Future<void> _initStream() async {
    if (!_alreadySetup) {
      _setupOnlineStatus();
      _setupOnKickOut();
      _setupChatRoomStatus();
      _setupSystemMessage();
      _setupOnMessage();

      _alreadySetup = true;
    }
  }

  /// 监听登录状态
  void _setupOnlineStatus() async {
    if (Platform.isAndroid) {
      final observer =
          await com_netease_nimlib_sdk_NIMSDK.getAuthServiceObserve();
      await observer.observeOnlineStatus(
        _AndroidCallback<com_netease_nimlib_sdk_StatusCode>(
          onEventListener: (status) => _onlineStatus.add(status.toStatusType()),
        ),
        true,
      );

      await observer.release__();
    } else if (Platform.isIOS) {
      final nim = await NIMSDK.sharedSDK();
      final loginManager = await nim.get_loginManager();
      await loginManager.addDelegate(
        _iOSCallback(
          onLoginStep: (step) => _onlineStatus.add(step.toStatusType()),
        ),
      );
      await nim.release__();
      await loginManager.release__();
    } else {
      throw '未实现的平台';
    }
  }

  /// 监听聊天室状态
  void _setupChatRoomStatus() async {
    if (Platform.isAndroid) {
      final observer = await com_netease_nimlib_sdk_NIMChatRoomSDK
          .getChatRoomServiceObserve();
      await observer.observeOnlineStatus(
        _AndroidCallback<
                com_netease_nimlib_sdk_chatroom_model_ChatRoomStatusChangeData>(
            onEventListener: (data) async {
          final roomId = await data.get_roomId();
          final status = await data.get_status();
          _chatRoomStatus.add(ChatRoomStatus(roomId, status.toStatusType()));
        }),
        true,
      );

      await observer.release__();
    } else if (Platform.isIOS) {
      final nim = await NIMSDK.sharedSDK();
      final manager = await nim.get_chatroomManager();
      await manager.addDelegate(
        _iOSCallback(onChatRoomStatus: _chatRoomStatus.add),
      );
      await nim.release__();
      await manager.release__();
    } else {
      throw '未实现的平台';
    }
  }

  /// 监听消息
  void _setupOnMessage() async {
    if (Platform.isAndroid) {
      final msgObserver =
          await com_netease_nimlib_sdk_NIMSDK.getMsgServiceObserve();
      await msgObserver.observeReceiveMessage(
        _AndroidCallback<List<com_netease_nimlib_sdk_msg_model_IMMessage>>(
          onEventListener: (messageList) {
            messageList.forEach((element) async {
              _message.add(await element.toMessage());
            });
          },
        ),
        true,
      );
      final chatroomObserver = await com_netease_nimlib_sdk_NIMChatRoomSDK
          .getChatRoomServiceObserve();
      await chatroomObserver.observeReceiveMessage(
        _AndroidCallback<
            List<com_netease_nimlib_sdk_chatroom_model_ChatRoomMessage>>(
          onEventListener: (messageList) {
            messageList.forEach((element) async {
              _message.add(await element.toMessage());
            });
          },
        ),
        true,
      );
      await msgObserver.release__();
      await chatroomObserver.release__();
    } else if (Platform.isIOS) {
      final nim = await NIMSDK.sharedSDK();
      final manager = await nim.get_chatManager();
      await manager.addDelegate(
        _iOSCallback(
          onMessageList: (messageList) {
            messageList.forEach((element) async {
              _message.add(await element.toMessage());
            });
          },
        ),
      );
      await nim.release__();
      await manager.release__();
    } else {
      throw '未实现的平台';
    }
  }

  /// 监听踢出聊天室事件
  void _setupOnKickOut() async {
    if (Platform.isAndroid) {
      final observer = await com_netease_nimlib_sdk_NIMChatRoomSDK
          .getChatRoomServiceObserve();
      await observer.observeKickOutEvent(
        _AndroidCallback<
                com_netease_nimlib_sdk_chatroom_model_ChatRoomKickOutEvent>(
            onEventListener: (data) async {
          final roomId = await data.getRoomId();
          final reason = await data.getReason();
          _kickOut.add(KickOut(roomId, reason.toKickOutReason()));
        }),
        true,
      );

      await observer.release__();
    } else if (Platform.isIOS) {
      final nim = await NIMSDK.sharedSDK();
      final manager = await nim.get_chatroomManager();
      await manager.addDelegate(_iOSCallback(onKickOut: _kickOut.add));

      await nim.release__();
      await manager.release__();
    } else {
      throw '未实现的平台';
    }
  }

  /// 监听系统消息
  void _setupSystemMessage() async {
//    if (Platform.isAndroid) {
//      final observer =
//          await com_netease_nimlib_sdk_NIMSDK_X.getSystemMessageObserver();
//      await observer.observeReceiveSystemMsg(
//        _AndroidCallback<com_netease_nimlib_sdk_msg_model_SystemMessage>(
//            onEventListener: (data) async {
//          final roomId = await data.getRoomId();
//          final reason = await data.getReason();
//          _kickOut.add(KickOut(roomId, reason.toKickOutReason()));
//        }),
//        true,
//      );
//
//      await observer.release__();
//    } else if (Platform.isIOS) {
//      final nim = await NIMSDK.sharedSDK();
//      final manager = await nim.get_systemNotificationManager();
//      await manager.addDelegate(_iOSCallback(onKickOut: _kickOut.add));
//
//      await nim.release__();
//      await manager.release__();
//    } else {
//      throw '未实现的平台';
//    }
  }

  /// 释放资源
  void dispose() {
    _onlineStatus.close();
    _message.close();
    _chatRoomStatus.close();
    _kickOut.close();
    _systemMessage.close();
  }
}

/// 登录登出
mixin _Auth on _NimStream {
  /// 登录
  ///
  /// [account] 用户账户
  /// [token] 登录密码
  /// sdk会自动连接服务器，传递用户信息，返回登录结果。
  /// 该操作中途可取消。如果因为网络比较差，或其他原因导致服务器迟迟没有返回，用户也没有主动取消，
  /// 在45秒后AbortableFuture的onFailed会被调用到。
  Future<void> login({@required String account, @required String token}) async {
    final completer = Completer<void>();
    debugPrint('此为付费插件, 请联系qq 382146139');

    // 登录完成后, 初始化一下监听流
    await _initStream();
  }

  /// 登出
  ///
  /// 如果用户手动登出，不再接收消息和提醒，开发者可以调用 logout 方法，该方法没有回调。
  Future<void> logout() async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }
}

/// 发送消息
mixin _SendMessage {
  /// 发送文字消息
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [type] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [content] 发送的文字内容
  Future<void> sendTextMessage({
    @required String sessionId,
    @required SessionType type,
    @required String content,
  }) async {
    final completer = Completer<void>();
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 发送图片消息
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [type] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [imageFile] 发送的图片文件
  Future<void> sendImageMessage({
    @required String sessionId,
    @required SessionType type,
    @required File imageFile,
  }) async {
    final completer = Completer<void>();
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 发送音频消息
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [type] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [audioFile] 发送的音频文件
  /// [duration] 音频时长
  Future<void> sendAudioMessage({
    @required String sessionId,
    @required SessionType type,
    @required File audioFile,
    @required Duration duration,
  }) async {
    final completer = Completer<void>();
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 发送视频消息
  ///
  /// [sessionId]   聊天对象ID
  /// [sessionType] 会话类型
  /// [file]        视频文件对象
  /// [duration]    视频文件持续时间
  /// [width]       视频宽度
  /// [height]      视频高度
  /// [displayName] 视频文件显示名，可以为空
  Future<void> sendVideoMessage({
    @required String sessionId,
    @required SessionType type,
    @required File videoFile,
    @required Duration duration,
    @required int width,
    @required int height,
    String displayName,
  }) async {
    final completer = Completer<void>();
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 发送文件消息
  ///
  /// [sessionId]   聊天对象ID
  /// [sessionType] 会话类型
  /// [file]        文件对象
  /// [displayName] 文件显示名，可以为空
  Future<void> sendFileMessage({
    @required String sessionId,
    @required SessionType type,
    @required File file,
    String displayName,
  }) async {
    final completer = Completer<void>();
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 发送位置消息
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [type] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [latitude] 纬度
  /// [longitude] 经度
  /// [address] 地址
  Future<void> sendLocationMessage({
    @required String sessionId,
    @required SessionType type,
    @required double latitude,
    @required double longitude,
    @required String address,
  }) async {
    final completer = Completer<void>();
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 发送自定义消息
  ///
  /// [sessionId]   聊天对象ID
  /// [sessionType] 会话类型
  /// [attachmentString] 自定义消息体
  Future<void> sendCustomMessage({
    @required String sessionId,
    @required SessionType type,
    @required String attachmentString,
  }) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }
}

/// 用户关系
mixin _UserRelationship {
  /// 获取我的好友列表
  Future<List<NimUser>> fetchMyFriendList() async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 获取免打扰列表
  Future<List<NimUser>> fetchMuteList() async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 请求添加好友
  ///
  /// [account] 对方账号
  /// [verifyType] 验证类型, 分为`直接添加`和`需要验证`
  /// [message] 请求信息
  Future<void> requestAddFriend({
    @required String account,
    @required VerifyType verifyType,
    String message = '',
  }) async {
    final completer = Completer<void>();
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 删除好友
  ///
  /// [account] 对方账户
  /// [removeAlias] 是否同时删除别名
  Future<void> deleteFriend(String account, {bool removeAlias = false}) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 处理添加好友申请(同意/拒绝)
  ///
  /// [account] 对方账户
  /// [agree] 是否同意
  Future<void> ackAddFriendRequest(
    String account, {
    @required bool agree,
  }) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 判断用户是好友
  Future<bool> isMyFriend(String account) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 加入黑名单
  Future<void> addToBlackList(String account) async {
    final completer = Completer<void>();
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 移出黑名单
  Future<void> removeFromBlackList(String account) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 判断用户是否在黑名单
  Future<bool> isInBlackList(String account) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 判断用户是免打扰
  Future<bool> isMute(String account) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 获取黑名单列表
  Future<List<NimUser>> fetchBlackList() async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 是否接收用户[account]的消息
  ///
  /// [mute] true为免打扰(静音)
  Future<void> setMute(String account, {@required bool mute}) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }
}

/// 聊天室
mixin _ChatRoom {
  /// 进入聊天室
  ///
  /// [chatRoomId] 聊天室id
  /// [retryCount] 重试次数
  Future<EnterChatRoomResult> enterChatRoom(
    String chatRoomId, {
    int retryCount = 3,
  }) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 退出聊天室
  Future<void> exitChatRoom(String chatRoomId) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }

  /// 获取聊天室信息
  Future<ChatRoomInfo> fetchChatRoomInfo(String chatRoomId) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }
}

/// 会话
mixin _Session {
  /// 获取会话列表
  Future<List<RecentSession>> fetchSessionList() async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }
}

/// 历史消息
mixin _HistoryMessage {
  /// 获取消息列表
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [sessionType] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [anchor] 以此消息为锚点获取下一页的消息列表
  /// [limit] 消息分页大小
  Future<List<Message>> fetchMessageList(
    String sessionId, {
    @required SessionType sessionType,
    Message anchor,
    int limit = 20,
  }) async {
    debugPrint('此为付费插件, 请联系qq 382146139');
  }
}
